@@ -13,14 +13,14 @@ sub import {
 #   License or the Artistic License, as specified in the Perl README file.
 # --------------------------------------------------------------------------- #
 
-use 5.006;
+use 5.008001;
 
 use strict;
 use warnings;
 
 require DBI;
 
-our $VERSION = '1.36';
+our $VERSION = '1.38';
 
 our $drh    = undef;    # will hold driver handle
 our $err    = 0;        # will hold any error codes
@@ -105,35 +105,26 @@ sub connect {
         return;
     }
     $attributes ||= {};
+
     if ($dbname && $DBD::Mock::AttributeAliasing) {
         # this is the DB we are mocking
         $attributes->{mock_attribute_aliases} = DBD::Mock::_get_mock_attribute_aliases($dbname);
         $attributes->{mock_database_name} = $dbname;
     }
 
-    # Need to protect AutoCommit from $dbh caching - RobK.
-    my $autocommit = 1;
-    if( exists $attributes->{ 'AutoCommit' } ) {
-        $autocommit = delete $attributes->{ 'AutoCommit' };
-    }
+    # holds statement parsing coderefs/objects
+    $attributes->{mock_parser} = [];
+    # holds all statements applied to handle until manually cleared
+    $attributes->{mock_statement_history} = [];
+    # ability to fake a failed DB connection
+    $attributes->{mock_can_connect} = 1;
+    # ability to make other things fail :)
+    $attributes->{mock_can_prepare} = 1;
+    $attributes->{mock_can_execute} = 1;
+    $attributes->{mock_can_fetch}   = 1;
 
-    my $dbh = DBI::_new_dbh($drh, {
-        Name                   => $dbname,
-        # holds statement parsing coderefs/objects
-        mock_parser            => [],
-        # holds all statements applied to handle until manually cleared
-        mock_statement_history => [],
-        # ability to fake a failed DB connection
-        mock_can_connect       => 1,
-        # ability to make other things fail :)
-        mock_can_prepare       => 1,
-        mock_can_execute       => 1,
-        mock_can_fetch         => 1,
-        # rest of attributes
-        %{ $attributes },
-    }) || return;
-
-    $dbh->STORE( 'AutoCommit' => $autocommit );
+    my $dbh = DBI::_new_dbh($drh, {Name => $dbname})
+        || return;
 
     return $dbh;
 }
@@ -395,21 +386,21 @@ sub selectcol_arrayref {
     # something went wrong, and so return undef.
     return undef unless defined $a_ref || ref($a_ref) ne 'ARRAY';
 
+    my @cols = 0;
+    if (ref $attrib->{Columns} eq 'ARRAY') {
+        @cols = map { $_ - 1 } @{$attrib->{Columns}};
+    }
+
     # if we do get something then we
     # grab all the columns out of it.
-    return [ map { $_->[0] } @{$a_ref} ]
+    return [ map { @$_[@cols] } @{$a_ref} ]
 }
 
-{
-    my %autocommit;
 sub FETCH {
-    my ( $dbh, $attrib ) = @_;
+    my ( $dbh, $attrib, $value ) = @_;
     $dbh->trace_msg( "Fetching DB attrib '$attrib'\n" );
-    if ($attrib eq 'AutoCommit') {
-        $dbh->trace_msg( "Fetching AutoCommit\n" );
-        return $autocommit{$dbh};
-    }
-     elsif ($attrib eq 'Active') {
+
+    if ($attrib eq 'Active') {
         return $dbh->{mock_can_connect};
     }
     elsif ($attrib eq 'mock_all_history') {
@@ -445,11 +436,14 @@ sub FETCH {
 sub STORE {
     my ( $dbh, $attrib, $value ) = @_;
     $dbh->trace_msg( "Storing DB attribute '$attrib' with '" . (defined($value) ? $value : 'undef') . "'\n" );
+
     if ($attrib eq 'AutoCommit') {
-        $autocommit{$dbh} = $value;
-        return $value;
+        # These are magic DBI values that say we can handle AutoCommit
+        # internally as well
+        $value = ($value) ? -901 : -900;
     }
-    elsif ( $attrib eq 'mock_clear_history' ) {
+
+    if ( $attrib eq 'mock_clear_history' ) {
         if ( $value ) {
             $dbh->{mock_statement_history} = [];
         }
@@ -534,11 +528,10 @@ sub STORE {
         return $dbh->SUPER::STORE($attrib, $value);
     }
   else {
-      $dbh->trace_msg("... storing non-driver attribute ($attrib) with value ($value) that DBI wont handle\n");
+      $dbh->trace_msg("... storing non-driver attribute ($attrib) with value ($value) that DBI won't handle\n");
       return $dbh->{$attrib} = $value;
   }
 }
-}
 
 sub DESTROY {
     my ($dbh) = @_;
@@ -568,6 +561,14 @@ use warnings;
 
 $DBD::Mock::st::imp_data_size = 0;
 
+sub bind_col {
+    my ($sth, $param_num, $ref, $attr) = @_;
+
+    my $tracker = $sth->FETCH( 'mock_my_history' );
+    $tracker->bind_col( $param_num, $ref );
+    return 1;
+}
+
 sub bind_param {
     my ($sth, $param_num, $val, $attr) = @_;
     my $tracker = $sth->FETCH( 'mock_my_history' );
@@ -668,7 +669,16 @@ sub fetch {
     $dbh->{mock_can_fetch}++ if $dbh->{mock_can_fetch} < 0;
 
     my $tracker = $sth->FETCH( 'mock_my_history' );
-    return $tracker->next_record;
+
+    my $record = $tracker->next_record;
+
+    if ( my @cols = $tracker->bind_cols() ) {
+        for my $i ( grep { ref $cols[$_] } 0..$#cols ) {
+            ${ $cols[$i] } = $record->[$i];
+        }
+    }
+
+    return $record;
 }
 
 sub fetchrow_array {
@@ -910,11 +920,14 @@ use warnings;
 my $connection;
 
 sub connect {
-    my $class = "DBD::Mock::Pool";
-    $class = shift unless ref($_[0]);
-    my ($driver_handle, $username, $password, $attributes) = @_;
-    $connection = bless $driver_handle->connect(), "DBD::Mock::Pool::db" unless $connection;
-    return $connection;
+    return $connection if $connection;
+
+    # according to the code before my tweaks, this could be a class
+    # name, but it was never used - DR, 2008-11-08
+    shift unless ref $_[0];
+
+    my $drh = shift;
+    return $connection = bless $drh->connect(@_), 'DBD::Mock::Pool::db';
 }
 
 package DBD::Mock::Pool::db;
@@ -983,6 +996,11 @@ sub num_params {
     return scalar @{$self->{bound_params}};
 }
 
+sub bind_col {
+    my ($self, $param_num, $ref) = @_;
+    $self->{bind_cols}->[$param_num - 1] = $ref;
+}
+
 sub bound_param {
     my ($self, $param_num, $value) = @_;
     $self->{bound_params}->[$param_num - 1] = $value;
@@ -994,6 +1012,11 @@ sub bound_param_trailing {
     push @{$self->{bound_params}}, @values;
 }
 
+sub bind_cols {
+    my $self = shift;
+    return @{$self->{bind_cols} || []};
+}
+
 sub bind_params {
     my ($self, @values) = @_;
     @{$self->{bound_params}} = @values;
@@ -1340,7 +1363,7 @@ To do that our test might look like:
 
   # Now query that history record to
   # see if our expectations match reality
-  cmp_ok(scalar(@{$history}), '==', 1, 'Correct number of statements executed' ;
+  is(scalar(@{$history}), 1, 'Correct number of statements executed' ;
 
   my $login_st = $history->[0];
   like($login_st->statement,
@@ -1348,7 +1371,7 @@ To do that our test might look like:
       'Correct statement generated' );
 
   my $params = $login_st->bound_params;
-  cmp_ok(scalar(@{$params}), '==', 1, 'Correct number of parameters bound');
+  is(scalar(@{$params}), 1, 'Correct number of parameters bound');
   is($params->[0], $login_name, 'Correct value for parameter 1' );
 
   # Reset the handle for future operations
@@ -1588,9 +1611,7 @@ This attribute can be used to set a current DBD::Mock::Session object. For more
 
 This attribute is incremented each time an INSERT statement is passed to C<prepare> on a per-handle basis. It's starting value can be set with  the 'mock_start_insert_id' attribute (see below).
 
-This attribute also can be used with an ARRAY ref parameter, it's behavior is slightly different in that instead of incrementing the value for every C<prepare> it will only increment for each C<execute>. This allows it to be used over multiple C<execute> calls in a single C<$sth>. It's usage looks like this:
-
-  $dbh->{mock_last_insert_id} = [ 'Foo', 10 ];
+  $dbh->{mock_start_insert_id} = 10;
 
   my $sth = $dbh->prepare('INSERT INTO Foo (foo, bar) VALUES(?, ?)');
 
@@ -1606,6 +1627,25 @@ For more examples, please refer to the test file F<t/025_mock_last_insert_id.t>.
 
 This attribute can be used to set a start value for the 'mock_last_insert_id' attribute. It can also be used to effectively reset the 'mock_last_insert_id' attribute as well.
 
+This attribute also can be used with an ARRAY ref parameter, it's behavior is slightly different in that instead of incrementing the value for every C<prepare> it will only increment for each C<execute>. This allows it to be used over multiple C<execute> calls in a single C<$sth>. It's usage looks like this:
+
+  $dbh->{mock_start_insert_id} = [ 'Foo', 10 ];
+  $dbh->{mock_start_insert_id} = [ 'Baz', 20 ];
+
+  my $sth1 = $dbh->prepare('INSERT INTO Foo (foo, bar) VALUES(?, ?)');
+
+  my $sth2 = $dbh->prepare('INSERT INTO Baz (baz, buz) VALUES(?, ?)');
+
+  $sth1->execute(1, 2);
+  # $dbh->{mock_last_insert_id} == 10
+
+  $sth2->execute(3, 4);
+  # $dbh->{mock_last_insert_id} == 20
+
+Note that DBD::Mock's matching of table names in 'INSERT' statements is fairly simple, so if your table names are quoted in the insert statement (C<INSERT INTO "Foo">) then you need to quote the name for C<mock_start_insert_id>:
+
+  $dbh->{mock_start_insert_id} = [ q{"Foo"}, 10 ];
+
 =item B<mock_add_parser>
 
 DBI provides some simple parsing capabilities for 'SELECT' statements to ensure that placeholders are bound properly. And typically you may simply want to check after the fact that a statement is syntactically correct, or at least what you expect.
